@@ -31,7 +31,7 @@
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
 
-#include "gsm-marshal.h"
+#include "gsm-system.h"
 #include "gsm-consolekit.h"
 
 #define CK_NAME      "org.freedesktop.ConsoleKit"
@@ -43,6 +43,8 @@
 #define CK_SEAT_INTERFACE    "org.freedesktop.ConsoleKit.Seat"
 #define CK_SESSION_INTERFACE "org.freedesktop.ConsoleKit.Session"
 
+#define GSM_CONSOLEKIT_SESSION_TYPE_LOGIN_WINDOW "LoginWindow"
+
 #define GSM_CONSOLEKIT_GET_PRIVATE(o)                                   \
         (G_TYPE_INSTANCE_GET_PRIVATE ((o), GSM_TYPE_CONSOLEKIT, GsmConsolekitPrivate))
 
@@ -51,21 +53,8 @@ struct _GsmConsolekitPrivate
         DBusGConnection *dbus_connection;
         DBusGProxy      *bus_proxy;
         DBusGProxy      *ck_proxy;
-        guint32          is_connected : 1;
 };
 
-enum {
-        PROP_0,
-        PROP_IS_CONNECTED
-};
-
-enum {
-        REQUEST_COMPLETED = 0,
-        LAST_SIGNAL
-};
-
-static guint signals[LAST_SIGNAL] = { 0 };
-
 static void     gsm_consolekit_class_init   (GsmConsolekitClass *klass);
 static void     gsm_consolekit_init         (GsmConsolekit      *ck);
 static void     gsm_consolekit_finalize     (GObject            *object);
@@ -82,59 +71,20 @@ static void     gsm_consolekit_on_name_owner_changed (DBusGProxy        *bus_pro
                                                       const char        *new_owner,
                                                       GsmConsolekit   *manager);
 
-G_DEFINE_TYPE (GsmConsolekit, gsm_consolekit, G_TYPE_OBJECT);
+static void gsm_consolekit_system_init (GsmSystemInterface *iface);
 
-static void
-gsm_consolekit_get_property (GObject    *object,
-                             guint       prop_id,
-                             GValue     *value,
-                             GParamSpec *pspec)
-{
-        GsmConsolekit *manager = GSM_CONSOLEKIT (object);
-
-        switch (prop_id) {
-        case PROP_IS_CONNECTED:
-                g_value_set_boolean (value,
-                                     manager->priv->is_connected);
-                break;
-
-        default:
-                G_OBJECT_WARN_INVALID_PROPERTY_ID (object,
-                                                   prop_id,
-                                                   pspec);
-        }
-}
+G_DEFINE_TYPE_WITH_CODE (GsmConsolekit, gsm_consolekit, G_TYPE_OBJECT,
+                         G_IMPLEMENT_INTERFACE (GSM_TYPE_SYSTEM,
+                                                gsm_consolekit_system_init))
 
 static void
 gsm_consolekit_class_init (GsmConsolekitClass *manager_class)
 {
         GObjectClass *object_class;
-        GParamSpec   *param_spec;
 
         object_class = G_OBJECT_CLASS (manager_class);
 
         object_class->finalize = gsm_consolekit_finalize;
-        object_class->get_property = gsm_consolekit_get_property;
-
-        param_spec = g_param_spec_boolean ("is-connected",
-                                           "Is connected",
-                                           "Whether the session is connected to ConsoleKit",
-                                           FALSE,
-                                           G_PARAM_READABLE);
-
-        g_object_class_install_property (object_class, PROP_IS_CONNECTED,
-                                         param_spec);
-
-        signals [REQUEST_COMPLETED] =
-                g_signal_new ("request-completed",
-                              G_OBJECT_CLASS_TYPE (object_class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GsmConsolekitClass, request_completed),
-                              NULL,
-                              NULL,
-                              g_cclosure_marshal_VOID__POINTER,
-                              G_TYPE_NONE,
-                              1, G_TYPE_POINTER);
 
         g_type_class_add_private (manager_class, sizeof (GsmConsolekitPrivate));
 }
@@ -232,11 +182,6 @@ gsm_consolekit_ensure_ck_connection (GsmConsolekit  *manager,
         is_connected = TRUE;
 
  out:
-        if (manager->priv->is_connected != is_connected) {
-                manager->priv->is_connected = is_connected;
-                g_object_notify (G_OBJECT (manager), "is-connected");
-        }
-
         if (!is_connected) {
                 if (manager->priv->dbus_connection == NULL) {
                         if (manager->priv->bus_proxy != NULL) {
@@ -336,28 +281,6 @@ gsm_consolekit_finalize (GObject *object)
         }
 }
 
-GQuark
-gsm_consolekit_error_quark (void)
-{
-        static GQuark error_quark = 0;
-
-        if (error_quark == 0) {
-                error_quark = g_quark_from_static_string ("gsm-consolekit-error");
-        }
-
-        return error_quark;
-}
-
-GsmConsolekit *
-gsm_consolekit_new (void)
-{
-        GsmConsolekit *manager;
-
-        manager = g_object_new (GSM_TYPE_CONSOLEKIT, NULL);
-
-        return manager;
-}
-
 static void
 emit_restart_complete (GsmConsolekit *manager,
                        GError        *error)
@@ -367,14 +290,13 @@ emit_restart_complete (GsmConsolekit *manager,
         call_error = NULL;
 
         if (error != NULL) {
-                call_error = g_error_new_literal (GSM_CONSOLEKIT_ERROR,
-                                                  GSM_CONSOLEKIT_ERROR_RESTARTING,
+                call_error = g_error_new_literal (GSM_SYSTEM_ERROR,
+                                                  GSM_SYSTEM_ERROR_RESTARTING,
                                                   error->message);
         }
 
-        g_signal_emit (G_OBJECT (manager),
-                       signals [REQUEST_COMPLETED],
-                       0, call_error);
+        g_signal_emit_by_name (G_OBJECT (manager),
+                               "request_completed", call_error);
 
         if (call_error != NULL) {
                 g_error_free (call_error);
@@ -390,23 +312,23 @@ emit_stop_complete (GsmConsolekit *manager,
         call_error = NULL;
 
         if (error != NULL) {
-                call_error = g_error_new_literal (GSM_CONSOLEKIT_ERROR,
-                                                  GSM_CONSOLEKIT_ERROR_STOPPING,
+                call_error = g_error_new_literal (GSM_SYSTEM_ERROR,
+                                                  GSM_SYSTEM_ERROR_STOPPING,
                                                   error->message);
         }
 
-        g_signal_emit (G_OBJECT (manager),
-                       signals [REQUEST_COMPLETED],
-                       0, call_error);
+        g_signal_emit_by_name (G_OBJECT (manager),
+                               "request_completed", call_error);
 
         if (call_error != NULL) {
                 g_error_free (call_error);
         }
 }
 
-void
-gsm_consolekit_attempt_restart (GsmConsolekit *manager)
+static void
+gsm_consolekit_attempt_restart (GsmSystem *system)
 {
+        GsmConsolekit *manager = GSM_CONSOLEKIT (system);
         gboolean res;
         GError  *error;
 
@@ -436,9 +358,10 @@ gsm_consolekit_attempt_restart (GsmConsolekit *manager)
         }
 }
 
-void
-gsm_consolekit_attempt_stop (GsmConsolekit *manager)
+static void
+gsm_consolekit_attempt_stop (GsmSystem *system)
 {
+        GsmConsolekit *manager = GSM_CONSOLEKIT (system);
         gboolean res;
         GError  *error;
 
@@ -596,10 +519,11 @@ get_current_seat_id (DBusConnection *connection)
         return seat_id;
 }
 
-void
-gsm_consolekit_set_session_idle (GsmConsolekit *manager,
-                                 gboolean       is_idle)
+static void
+gsm_consolekit_set_session_idle (GsmSystem *system,
+                                 gboolean   is_idle)
 {
+        GsmConsolekit *manager = GSM_CONSOLEKIT (system);
         gboolean        res;
         GError         *error;
         char           *session_id;
@@ -708,9 +632,10 @@ seat_can_activate_sessions (DBusConnection *connection,
         return can_activate;
 }
 
-gboolean
-gsm_consolekit_can_switch_user (GsmConsolekit *manager)
+static gboolean
+gsm_consolekit_can_switch_user (GsmSystem *system)
 {
+        GsmConsolekit *manager = GSM_CONSOLEKIT (system);
         GError  *error;
         char    *seat_id;
         gboolean ret;
@@ -737,9 +662,10 @@ gsm_consolekit_can_switch_user (GsmConsolekit *manager)
         return ret;
 }
 
-gboolean
-gsm_consolekit_can_restart (GsmConsolekit *manager)
+static gboolean
+gsm_consolekit_can_restart (GsmSystem *system)
 {
+        GsmConsolekit *manager = GSM_CONSOLEKIT (system);
         gboolean res;
 	gboolean can_restart;
         GError  *error;
@@ -771,9 +697,10 @@ gsm_consolekit_can_restart (GsmConsolekit *manager)
 	return can_restart;
 }
 
-gboolean
-gsm_consolekit_can_stop (GsmConsolekit *manager)
+static gboolean
+gsm_consolekit_can_stop (GsmSystem *system)
 {
+        GsmConsolekit *manager = GSM_CONSOLEKIT (system);
         gboolean res;
 	gboolean can_stop;
         GError  *error;
@@ -804,7 +731,7 @@ gsm_consolekit_can_stop (GsmConsolekit *manager)
 	return can_stop;
 }
 
-gchar *
+static gchar *
 gsm_consolekit_get_current_session_type (GsmConsolekit *manager)
 {
         GError *gerror;
@@ -871,33 +798,40 @@ out:
 	return ret;
 }
 
-
-GsmConsolekit *
-gsm_get_consolekit (void)
+static gboolean
+gsm_consolekit_is_login_session (GsmSystem *system)
 {
-        static GsmConsolekit *manager = NULL;
-
-        if (manager == NULL) {
-                manager = gsm_consolekit_new ();
-        }
-
-        return g_object_ref (manager);
-}
-
-gboolean
-gsm_consolekit_is_current_session_login (void)
-{
-        GsmConsolekit *consolekit;
+        GsmConsolekit *consolekit = GSM_CONSOLEKIT (system);
         char *session_type;
         gboolean ret;
 
-        consolekit = gsm_get_consolekit ();
         session_type = gsm_consolekit_get_current_session_type (consolekit);
 
         ret = (g_strcmp0 (session_type, GSM_CONSOLEKIT_SESSION_TYPE_LOGIN_WINDOW) == 0);
 
-        g_object_unref (consolekit);
         g_free (session_type);
 
         return ret;
 }
+
+static void
+gsm_consolekit_system_init (GsmSystemInterface *iface)
+{
+        iface->can_switch_user = gsm_consolekit_can_switch_user;
+        iface->can_stop = gsm_consolekit_can_stop;
+        iface->can_restart = gsm_consolekit_can_restart;
+        iface->attempt_stop = gsm_consolekit_attempt_stop;
+        iface->attempt_restart = gsm_consolekit_attempt_restart;
+        iface->set_session_idle = gsm_consolekit_set_session_idle;
+        iface->is_login_session = gsm_consolekit_is_login_session;
+}
+
+GsmConsolekit *
+gsm_consolekit_new (void)
+{
+        GsmConsolekit *manager;
+
+        manager = g_object_new (GSM_TYPE_CONSOLEKIT, NULL);
+
+        return manager;
+}
