@@ -34,7 +34,7 @@ enum {
 
 static guint signals[LAST_SIGNAL] = { 0 };
 
-G_DEFINE_INTERFACE (GsmSystem, gsm_system, G_TYPE_INTERFACE);
+G_DEFINE_INTERFACE (GsmSystem, gsm_system, G_TYPE_OBJECT)
 
 static void
 gsm_system_default_init (GsmSystemInterface *iface)
